#ifndef GAME_TREE_H
#define GAME_TREE_H

#include "BaseClasses.h"
#include <time.h>

namespace game
{

template <class GameState>
class GameTreeNode
{
public:
	GameTreeNode()
	{
		node.init();
		//children.clear();
		depth = 0;
		isLeaf = true;
	}

	~GameTreeNode()
	{
		clear();		
	}

	void clear()
	{
		node.clear();
		//children.clear();
		depth = 0;
		isLeaf = true;
	}

	GameState node;
	std::vector<GameTreeNode<GameState>* > children;
	unsigned int depth;
	bool isLeaf;

};

template <class GameState, class StateEvaluator>
class GameTree
{
public:
	GameTree()
	{
		//root->clear();
		treeBuilt = false;
	}
	~GameTree()
	{
		root->clear();
		treeBuilt = false;
	}

	void setEvaluator(const StateEvaluator &_evaluator){evaluator = _evaluator;}

	void build(const GameState &initState, int maxDepth=0)
	{
		root = new GameTreeNode<GameState>();
		root->node = initState;
		build_r(root, maxDepth);
		treeBuilt = true;
	}

	void clear()
	{
		root->clear();
		treeBuilt = false;
	}

	void minimax(GameState &best)
	{
		if(treeBuilt)
		{
			/*if(best.isOpening())
			{
				int middle = root->children.size() / 4;
				srand ( time(NULL) );
				int randIndex = rand() % (root->children.size()/2) + middle;
				best = root->children[randIndex]->node;
			}
			else
			{*/
				double maxVal = -1.0*DBL_MAX;
				int maxIdx = 0;
				for(size_t i = 0; i < root->children.size(); ++i)
				{
					double max = minTree(root->children[i]);
					if(max > maxVal)
					{
						maxVal = max;
						maxIdx = i;
					}
				}
				best = root->children[maxIdx]->node;
			/*}*/
		}
	}

private:

	GameTreeNode<GameState> *root;
	StateEvaluator evaluator;
	bool treeBuilt;

	//to build the tree recursively
	void build_r(GameTreeNode<GameState> *node, int maxDepth)
	{
		std::vector<GameState> childStates;
		node->node.spawn(childStates);
		if(childStates.size() == 0 || node->depth == maxDepth)
			node->isLeaf = true;
		else
		{
			node->isLeaf = false;
			node->children.resize(childStates.size());
			for(size_t i = 0; i < node->children.size(); ++i)
			{
				node->children[i] = new GameTreeNode<GameState>();
				node->children[i]->node = childStates[i];
				node->children[i]->depth = node->depth + 1;
				build_r(node->children[i], maxDepth);
			}
		}	
	}

	double minTree(GameTreeNode<GameState> *node)
	{
		if(node->isLeaf)
		{
			return evaluator.funct(node->node);
		}
		else
		{
			double minVal = DBL_MAX;
			for(size_t i = 0; i < node->children.size(); ++i)
			{
				double max = maxTree(node->children[i]);
				if(max < minVal)
					minVal = max;
			}
			return minVal;
		}
	}

	double maxTree(GameTreeNode<GameState> *node)
	{
		if(node->isLeaf)
		{
			return evaluator.funct(node->node);
		}
		else
		{
			double maxVal = -1.0*DBL_MAX;
			for(size_t i = 0; i < node->children.size(); ++i)
			{
				double min = minTree(node->children[i]);
				if(min > maxVal)
					maxVal = min;
			}
			return maxVal;
		}
	}
};

}//namespace game

#endif