#ifndef PLAYER_H
#define PLAYER_H

#include <vector>
#include <memory>
#include "BaseClasses.h"
#include "GameTree.h"
#include <iostream>

	


namespace game
{

template <class GameState, class StateEvaluator>
class Player
{
public:
	virtual bool playTurn(GameState &state)=0;
	virtual void setDepth(const int _maxDepth)=0;
	virtual void setEvaluator(const StateEvaluator &_evaluator){evaluator = _evaluator;};
	StateEvaluator getEvaluator(){return evaluator;};
	virtual int getDepth()=0;

	StateEvaluator evaluator;
};

template <class GameState, class StateEvaluator>
class HumanPlayer : public Player<GameState, StateEvaluator>
{
public:
	HumanPlayer(){};
	~HumanPlayer(){};
	HumanPlayer(const HumanPlayer &cpy){};

	bool playTurn(GameState &state)
	{
		if(state.endState())
		{
			std::cout << "You Lost!" << std::endl;
			return false;
		}
		std::cout << "Enter selection: ";
		int selection;
		std::cin >> selection;
		GameState nextState;
		while(!state.spawn(nextState, selection-1))
		{
			std::cout << "Incorrect option" << std::endl;
			std::cout << "Enter selection: ";
			std::cin >> selection;
		}

		state = nextState;
		return true;
	};

	void setDepth(const int _maxDepth){};
	int getDepth(){return 0;};
};

template <class GameState, class StateEvaluator>
class AIPlayer : public Player<GameState, StateEvaluator>
{
public:
	AIPlayer(){maxDepth = 0;};
	~AIPlayer(){};
	AIPlayer(const AIPlayer &cpy){maxDepth = cpy.maxDepth;}
	AIPlayer(const int _maxDepth){maxDepth = _maxDepth;}

	void setDepth(const int _maxDepth){maxDepth = _maxDepth;}
	void setEvaluator(const StateEvaluator &_evaluator){evaluator = _evaluator; tree.setEvaluator(_evaluator);}
	int getDepth(){return maxDepth;}

	bool playTurn(GameState &state) 
	{
		if(state.endState())
		{
			std::cout << "AI Lost!" << std::endl;
			return false;
		}

		tree.build(state, maxDepth);
		tree.minimax(state);
		return true;
	}

private:
	GameTree<GameState, StateEvaluator> tree;
	int maxDepth;

};

}//namespace game

#endif